!
! GbjToolsInterface.gs
!
! Class used to provide needed support for the GBJ Tools as of GBJ30 and later.
!
doit
Object subclass: 'GbjToolsInterface'
  instVarNames: #( classOrganizer)
  classVars: #()
  classInstVars: #()
  poolDictionaries: #()
  inDictionary: Globals
%
!
! Remove existing behavior from GbjToolsInterface
! (Not currently used since we're creating a new class version above)
! (Re-enable if we move the class creation code elsewhere)
!
! doit
! GbjToolsInterface removeAllMethods.
! GbjToolsInterface class removeAllMethods.
!%
!
! ------------------- Class methods for GbjToolsInterface
category: 'private-debugging'
classmethod: GbjToolsInterface
generateJavaCode

    " Generates java code for GbjKernelObjects.java.
      Should only be run on an out-of-box extent.
      Creates file classes.txt to hold results. "

            | x y z cls ltrm  |
            x := Globals values select: [:e | e isBehavior ].
            x := x reject: [:e | (e name at: 1) = $G and: [(e name at: 2) = $b and: [(e name at: 3) = $j]]].
            x := x reject: [:e | (e name at: 1) = $G and: [(e name at: 2) = $s and: [(e name at: 3) = $b]]].
            x := x reject: [:e | (e name at: 1) = $H and: [(e name at: 2) = $T and: [(e name at: 3) = $T and: [(e name at: 4) = $P]]]].
            x := x asSortedCollection: [:a:b | a name < b name].
            y := Array withAll: x.
            y := y collect: [:e | e name].
            z := String new.
            ltrm := String new.
            ltrm addAll: '<br>'.
            ltrm add: Character lf; add: Character tab.
            ltrm addAll: ' * '.
            y do: [:n |
                    cls := Globals at: n.
                    z tab; addAll: '/'; addAll: '**'; lf.
                    (cls description notNil and: [cls description detailsAboutClass notNil]) ifTrue: [
                        z tab; addAll: ' * '; addAll: (cls description detailsAboutClass copyReplaceAll: (String with: Character lf) with: ltrm); lf ]
                    ifFalse: [
                        z tab; addAll: ' * refers to a class in GemStone with the same name'; lf ].
                    z tab; addAll: ' *'; add: $/; lf.
                    z tab; addAll: 'public GbjObject '; addAll: n;
                    addAll: ';'; lf ].

            z tab; addAll: 'public GbjKernelObjects(GbjSession session) { '; lf;
              tab; tab; addAll: 'maxSmallInteger = session.doit("SmallInteger maximumValue").longValue( );';lf;
              tab; tab; addAll: 'minSmallInteger = session.doit("SmallInteger minimumValue").longValue( );'; lf;
              tab; tab; addAll: 'Illegal = GbjObject.withOop(0, session);'; lf;
              tab; tab; addAll: 'Zero = session.doit("0");'; lf;
              tab; tab; addAll: 'One = session.doit("1");'; lf;
              tab; tab; addAll: 'True = session.doit("true");'; lf;
              tab; tab; addAll: 'False = session.doit("false");'; lf;
              tab; tab; addAll: 'Nil = session.doit("nil");'; lf.
            z tab; tab; addAll: 'GemStoneError = session.doit("GemStoneError");'; lf.
            y do: [:n |
                    z tab; tab; addAll: n;
                    addAll: ' = session.doit("';
                    addAll: n;
                    addAll: '");'; lf; yourself ].
            z tab; addAll: '}'; lf; lf; lf.
            z toServerTextFile: 'classes.txt'.
%
category: 'private-debugging'
classmethod: GbjToolsInterface
selectionIn: source fromStepPoint: aStepPoint offsets: offsets
	| startOffset endOffset found limit |

	aStepPoint > offsets size
	  ifTrue: [
		"Stepping out of an inner activation can leave the steppoint pointing past the end of the offsets array"
		endOffset := source size.
		startOffset := endOffset - 1]
	  ifFalse: [
		startOffset := offsets at: aStepPoint.
		aStepPoint = offsets size
			ifTrue: [limit := source size]
			ifFalse: [limit := offsets at: (aStepPoint + 1)].
		limit < startOffset
			ifTrue: [limit := source size].
		endOffset := startOffset + 1.
		found := false.
		[ (endOffset <= limit) _and: [found not] ] whileTrue: [
			found := (source at: endOffset) isSeparator.
			found ifFalse: [ endOffset := endOffset + 1 ]]].
	^Array with: startOffset with: endOffset - 1
%
category: 'private-debugging'
classmethod: GbjToolsInterface
stackInfoToLevel: aLevel
	^ (self _stackInfoToLevel: aLevel)
%
category: 'private-debugging'
classmethod: GbjToolsInterface
_gsiStackReportFromLevel: startLevel toLevel: stopLevel

	| result frameContents |
	result := Array new.
	startLevel to: stopLevel do: [:j | 
  		frameContents := GsProcess _frameContentsAt: j .
                "GBJ30: remove reference to GbjMessageProcessor
  		(frameContents == nil _or: [(frameContents at: 1) _inClass == GbjMessageProcessor])
                "
  		(frameContents == nil)
			ifTrue:[^result]
    			ifFalse:[
     		 		result addLast:(GsProcess _reportWithFrameContents: frameContents) ;
             				addLast:(GsProcess _gsiDetailedReportWithFrameContents: frameContents
			   	 forLevel: j )]].
	^ result
%
category: 'private-debugging'
classmethod: GbjToolsInterface
_stackInfoToLevel: aLevel
	" Private - Answer an array of information about the GemStone execution state of the
	  process in aGsError.  The first entry is another array (see GS class GsProcess, method
	  #_gsiStackReportFromLevel:toLevel:, for more details).  The second entry is an array
	  containing selected fields from the first entry that we know we will need as local
	  objects (this is a performance hack to reduce round-trips to GemStone). "
	"GbjToolsInterface _stackInfoToLevel: 10"

	| className aProtocol aSelector aMethod methodClass frameId fullEntry stackReport stackReportExtract quickStepPoint sourceOffsets sourceString range |
	stackReport := self _gsiStackReportFromLevel: 7 toLevel: aLevel.
	stackReportExtract := OrderedCollection new: stackReport size * 10.
	1 to: stackReport size / 2 do: [:index |
		frameId := stackReport at: index * 2 - 1.
		fullEntry := stackReport at: index * 2.
		aMethod := fullEntry at: 1.
		methodClass := aMethod _inClass.
                "GBJ30: not needed?
		methodClass == GbjMessageProcessor ifTrue: [^stackReportExtract].
                "
		stackReportExtract add: frameId.
		stackReportExtract add: aMethod. "gsMethod"
		stackReportExtract add: methodClass. "method class"
		className := (methodClass == nil) ifTrue: [#Unknown] ifFalse: [methodClass name].
		stackReportExtract add: className.
		stackReportExtract add: (fullEntry at: 3). "self"
		aSelector := fullEntry at: 4.
		stackReportExtract add: aSelector. "selector"
		aProtocol := (methodClass == nil) ifTrue: [#Unknown] ifFalse: [methodClass _categoryOfSelector: aSelector].
		stackReportExtract add: aProtocol.
		quickStepPoint := fullEntry at: 5.
		(quickStepPoint == nil) ifTrue: [
			quickStepPoint := aMethod _stepPointForIp: (fullEntry at: 10) level: index * 2 quick: false].
		sourceOffsets := fullEntry at: 6.
		sourceString := fullEntry at: 9.
                ((quickStepPoint == nil) _or: [quickStepPoint = 0 _or: [(sourceOffsets == nil) _or: [sourceOffsets isEmpty]]])
                    ifTrue: [ stackReportExtract add: -1.  stackReportExtract add: -1]
                    ifFalse: [
			range := self selectionIn: sourceString fromStepPoint: quickStepPoint offsets: sourceOffsets.
			stackReportExtract add: range first.  stackReportExtract add: range last].
		stackReportExtract add: (fullEntry at: 7). "argAndTempNames"
		stackReportExtract add: (fullEntry at: 8)]. "argAndTempValues"
	^stackReportExtract
%
category: 'private-debugging'
classmethod: GbjToolsInterface
generateGbjKernelObjectsCode: file

" Generates code for kernel class to oop map in GbjKernalObjects.java.
  (will still need to do some manual cleanup)
"

| classes classNames |

file log: '		// The following auto-generated by GS/S method'.
file log: '		// GbjToolsInterface>>generateGbjKernelObjectsCode:'.
file log: '		Illegal = new GbjObject(1, session);'.
file log: '		Zero = new GbjObject(' , 
			(0 asOop asString) , ', session);'.
file log: '		One = new GbjObject(' ,
			(1 asOop asString) , ', session);'.
file log: '		True = new GbjObject(' ,
			(true asOop asString) , ', session);'.
file log: '		False = new GbjObject(' ,
			(false asOop asString) , ', session);'.
file log: '		Nil = new GbjObject(' ,
			(nil asOop asString) , ', session);'.

classes := (Globals values select: [:x | x isKindOf: Behavior]) asArray.
classNames := (classes collect: [:x | x name ]).
classNames add: #GemStoneError. "Need to add manually since not a class"
classNames := classNames sortAscending.

classNames do: [:name |
   file log: '		' , name , ' = new GbjObject(' , 
             (((Globals at: name) asOop) asString) , ', session);' ].

file log: '		// end auto-generated code'.
%
! ------------------- Instance methods for GbjToolsInterface
category: 'private-browsing'
method: GbjToolsInterface
acceptClassText: aString oldClass: oldClass
	^ (self _acceptClassText: aString oldClass: oldClass)
%
category: 'private-browsing'
method: GbjToolsInterface
addDict: aString at: index
	| newDict dictName symList |
	"Create a new SymbolDictionary and populate it with it's naming Association."
	dictName := aString asSymbol.
	symList := System myUserProfile symbolList.
	((newDict := symList detect: [:each | each name = dictName] ifNone: [nil]) ~~ nil) ifTrue: [
		^newDict].
	newDict := SymbolDictionary new.
	newDict at: dictName put: newDict.

	"Insert the dictionary at the specified index of the user's symbol list and
	 add it to our internal representation."
	System myUserProfile insertDictionary: newDict at: index.

	"Return the new GemStone SymbolDictionary object."
	^newDict
%
category: 'private11-browsing'
method: GbjToolsInterface
addProtocol: aProtocol toClass: aClass
	(aClass categoryNames includes: aProtocol asSymbol) ifFalse: [
		aClass addCategory: aProtocol]
%
category: 'private-filing out'
method: GbjToolsInterface
asArrayOfLines: aString
	| result cr lf pos lastPos size thisChar |

	"Initialize"
	cr := 13 asCharacter.
	lf := 10 asCharacter.
	result := OrderedCollection new.
	pos := 0.
	lastPos := 1.
	size := aString size.

	[ pos < size ] whileTrue: [
		pos := pos + 1.
		thisChar := aString at: pos.
		(thisChar == cr _or: [ thisChar == lf _or: [ pos = size ]]) ifTrue: [
			"Treat cr-lf as a single newline"
			"(pos < size and: [ thisChar == cr and: [ (aString at: pos + 1) == lf ]])
				ifTrue: [ pos := pos + 1 ]."
			result add: (aString copyFrom: lastPos to: pos).
			lastPos := pos + 1.
			].
		].

	(thisChar == cr _or: [ thisChar == lf ]) ifTrue: [
		"Treat trailing newline as a new blank line"
		result add: aString class new
		].

	^result asArray
%
category: 'private-searching'
method: GbjToolsInterface
asGSPattern: initialString
	"Private.  GBS.  Convert <initialString> which possibly contains wild-card characters * and ? into a pattern suitable
	for use with the GemStone string matching facilities in method <matchPattern:>."

	| pattern stream aString |
	pattern := OrderedCollection new.
	initialString isEmpty ifTrue: [^pattern].
	stream := WriteStream on: (String new: 100).
	1 to: initialString size do: [:i |
		(#($* $?) includes: (initialString at: i))
			ifTrue: [
				(aString := stream contents) isEmpty not ifTrue: [pattern addLast: aString].
				stream reset.
				pattern addLast: (initialString at: i)]
			ifFalse: [stream nextPut: (initialString at: i)]].
	(aString := stream contents) isEmpty not ifTrue: [pattern addLast: aString].
	^pattern asArray
%
category: 'private11-breakpoints'
method: GbjToolsInterface
breakpointInfo
	^ self _breakpointInfo
%
category: 'private11-misc'
method: GbjToolsInterface
classFor: anObject
	^anObject class
%
category: 'private11-misc'
method: GbjToolsInterface
classHistoryFor: aClass
	^aClass classHistory
%
category: 'private-browsing'
method: GbjToolsInterface
classInfoForClass: aClass
	| classInfo |
	classInfo := OrderedCollection new.
	self classInfoForClass: aClass into: classInfo.
	^classInfo
%
category: 'private-browsing'
method: GbjToolsInterface
classInfoForClass: aClass into: classInfo
	classInfo add: aClass.
	classInfo add: aClass name.
	classInfo add: (aClass classHistory indexOf: aClass).
	classInfo add: aClass classHistory size.
	classInfo add: aClass class
%
category: 'private-browsing'
method: GbjToolsInterface
classInfoForDictionary: aDictionary
	^ (self _classInfoForDictionary: aDictionary)
%
category: 'private-browsing'
method: GbjToolsInterface
classNamed: aString
	| object |
	object := System myUserProfile objectNamed: aString asSymbol.
	object isBehavior
		ifTrue: [^object]
		ifFalse: [^nil]
%
category: 'private-inspecting'
method: GbjToolsInterface
classNameFor: anObject
	^anObject class name
%
category: 'private-searching'
method: GbjToolsInterface
classNamesMatching: aString
	"Private.  Return a list of class names that match <aString> which may contain wild-card characters * and ?."

	| matches pattern |
	matches := OrderedCollection new.
	pattern := self asGSPattern: aString.
	System myUserProfile symbolList do: [:aDict |
		aDict associationsDo: [:assoc |
			(assoc value isKindOf: Class) ifTrue: [
				(assoc value name matchPattern: pattern) ifTrue: [
					matches add: assoc value name]]]].
	^matches asSortedCollection asArray
%
category: 'private-accessing'
method: GbjToolsInterface
classOrganizer
	(classOrganizer == nil) ifTrue: [
		classOrganizer := ClassOrganizer new].
	^classOrganizer
%
category: 'private11-var refs'
method: GbjToolsInterface
classVarNamesInClass: aClass
	^aClass allClassVarNames
%
category: 'private-browsing'
method: GbjToolsInterface
classVersionInfoForHistory: hist
	^ (self _classVersionInfoForHistory: hist)
%
category: 'private11-breakpoints'
method: GbjToolsInterface
closestOffsetIn: aMethod at: textPosition
	"self new closestOffsetIn: (self compiledMethodAt: #closestOffsetIn:at:) at: 100"
	| offsets offset diff index |
	index := 1.
	offsets := aMethod _sourceOffsets.
	diff := 65535.
	1 to: offsets size do: [:i |
		offset := offsets at: i.
		(offset - textPosition) abs < diff ifTrue: [
			index := i.
			diff := (offset - textPosition) abs]].
	^index
%
category: 'private11-misc'
method: GbjToolsInterface
commentForClass: aClass
	| description comment |
	description := aClass description.
	(description ~~ nil) 
		ifTrue: [comment := description detailsAboutClass]
		ifFalse: [comment := String new].
	^comment
%
category: 'private-browsing'
method: GbjToolsInterface
compile: aString class: aClass category: aCategory
	^ (self _compile: aString class: aClass category: aCategory)
%
category: 'private11-debugging'
method: GbjToolsInterface
continueProcess: aProcess
        ^aProcess _continue
%
category: 'private-filing out'
method: GbjToolsInterface
crsOnly: aString

	"Private.  GBS.  Return a copy of the receiver which has substituted a single CR for occurrences of LF or CRLF,  and removed occurrences of the null character."

	| cr lf rs ws char |
	cr := 13 asCharacter.
	lf := 10 asCharacter.
	rs := ReadStream on: aString.
	ws := WriteStream on: (String new: aString size).
	[ rs atEnd ] whileFalse: [
		char := rs next.
		char == cr ifTrue: [ rs peek == lf ifTrue: [ rs next ]].
		char == lf
			ifTrue: [
				ws nextPut: cr
				]
			ifFalse: [
				char asInteger > 0 ifTrue: [ ws nextPut: char ]
				]
		].
	^aString class withAll: ws contents
%
category: 'private11-misc'
method: GbjToolsInterface
definitionForClass: aClass
	^aClass definition
%
category: 'private-browsing'
method: GbjToolsInterface
dictionaryInfo
	^ self _dictionaryInfo
%
category: 'private-browsing'
method: GbjToolsInterface
dictionaryInfoForClass: aClass
	"GbjToolsInterface dictionaryInfoForClass: Object"

	^ (self _dictionaryInfoForClass: aClass)
%
category: 'private11-misc'
method: GbjToolsInterface
dictionaryNames
	^System myUserProfile symbolList asOrderedCollection collect: [:each | each name]
%
category: 'private11-breakpoints'
method: GbjToolsInterface
disableAllBreaks
	GsNMethod _disableAllBreaks
%
category: 'private11-breakpoints'
method: GbjToolsInterface
disableBreakpoint: stepPoint inMethod: aMethod
	aMethod disableBreakAtStepPoint: stepPoint
%
category: 'private11-misc'
method: GbjToolsInterface
doesCollection: aCollection include: element
	^aCollection includes: element
%
category: 'private11-breakpoints'
method: GbjToolsInterface
enableAllBreaks
	GsNMethod _enableAllBreaks
%
category: 'private11-breakpoints'
method: GbjToolsInterface
enableBreakpoint: stepPoint inMethod: aMethod
	aMethod setBreakAtStepPoint: stepPoint
%
category: 'private-browsing'
method: GbjToolsInterface
errorInfoFrom: errorArray
	^ (self _errorInfoFrom: errorArray)
%
category: 'private-executing'
method: GbjToolsInterface
evaluate: aString inContext: context names: names values: values printing: aBoolean
	| symList result |
	symList := self symbolListWithNames: names values: values.
	result := aString evaluateInContext: context symbolList: symList.
	^aBoolean
		ifTrue: [result printString]
		ifFalse: [result]
%
category: 'private-filing in'
method: GbjToolsInterface
fileInString: aString
	^GbjFileinReader onString: aString
%
category: 'private-filing out'
method: GbjToolsInterface
fileOutClass: aClass
	"GbjToolsInterface new fileOutClass: Array"

	| aStream |
	aStream := WriteStream on: (String new: 1024).
	self fileOutClass: aClass onFile: aStream.
	^aStream contents
%
category: 'private-filing out'
method: GbjToolsInterface
fileOutClass: aClass onFile: aFile
	"This will file out the given symbol dictionary." 

	self writeFileOutHeaderOn: aFile.
	aFile nextPutAll: aClass thisClass fileOutClass
%
category: 'private-filing out'
method: GbjToolsInterface
fileOutDict: aDict
	"GbjToolsInterface new fileOutDict: UserGlobals"

	| aStream |
	aStream := WriteStream on: (String new: 1024).
	self fileOutDict: aDict onFile: aStream.
	^aStream contents
%
category: 'private-filing out'
method: GbjToolsInterface
fileOutDict: aDict onFile: aFile
	"This will file out the given symbol dictionary." 

	| dictName |
	dictName := aDict name.	 
	self writeFileOutHeaderOn: aFile.
	aFile nextPutAll: '!'; lf. 
	aFile nextPutAll: '! SymbolDictionary '''; 
		nextPutAll: dictName; 
		nextPutAll: ''''; lf. 
	aFile nextPutAll: '!'; lf. 
	aFile nextPutAll: 'run'; lf. 
	aFile nextPutAll: '| symList newDict |'; lf. 
	aFile nextPutAll: 'symList := System myUserProfile symbolList.'; lf. 
	aFile nextPutAll: 'symList do: [ :element |'; lf. 
	aFile nextPutAll: '    (element includesKey: #'; 
		nextPutAll: dictName; 
		nextPutAll: ')'; lf. 
	aFile nextPutAll: '        ifTrue: [ ^element ]'; lf. 
	aFile nextPutAll: '].'; lf. 
	aFile nextPutAll: 'newDict := SymbolDictionary new.'; lf. 
	aFile nextPutAll: 'newDict at: #'; 
		nextPutAll: dictName; 
		nextPutAll: ' put: newDict.'; lf. 
	aFile nextPutAll: 'System myUserProfile insertDictionary: newDict at: 1.'; lf. 
	aFile nextPutAll: '^newDict'; lf. 
	aFile nextPutAll: '%'; lf. 

	self classOrganizer fileOutClassesAndMethodsInDictionary: aDict on: aFile
%
category: 'private-filing out'
method: GbjToolsInterface
fileOutMethod: aSelector inClass: aClass
	"GbjToolsInterface new fileOutMethod: 'add:' inClass: Array"

	| aStream |
	aStream := WriteStream on: (String new: 1024).
	self fileOutMethod: aSelector asSymbol inClass: aClass onFile: aStream.
	^aStream contents
%
category: 'private-filing out'
method: GbjToolsInterface
fileOutMethod: aSelector inClass: aClass onFile: aFile
	"This will file out the given symbol dictionary." 

	| nonMeta sourceCode newSource |
	nonMeta := aClass thisClass.
	self writeFileOutHeaderOn: aFile.	
	aFile
		nextPutAll: 'category: ''' , (aClass categoryOfSelector: aSelector);
		nextPutAll: ''''; lf. 
	sourceCode := aClass sourceCodeAt: aSelector.
	aClass isMeta
		ifTrue: [ aFile nextPutAll: 'classmethod: ' , nonMeta name; lf ] 
		ifFalse: [ aFile nextPutAll: 'method: ' , nonMeta name; lf ]. 
	newSource := sourceCode.
	aFile nextPutAll: newSource. 
	((newSource at: (newSource size)) = Character lf)
		ifFalse: [ aFile lf ].
	aFile nextPutAll: '%'; lf
%
category: 'private-browsing'
method: GbjToolsInterface
hierarchyListForClass: aClass
	^ (self _hierarchyListForClass: aClass)
%
category: 'private-browsing'
method: GbjToolsInterface
implementorInfoForSelector: aSelector
	"self new implementorInfoForSelector: 'copy'"

	^ (self _implementorInfoForSelector: aSelector)
%
category: 'private-browsing'
method: GbjToolsInterface
infoForClassNamed: aString
	^ (self _infoForClassNamed: aString)
%
category: 'private11-debugging'
method: GbjToolsInterface
initialInvocationOfMethodAtLevel: aLevel
	^aLevel
%
category: 'private-browsing'
method: GbjToolsInterface
instancesOf: aClass
	"self new instancesOf: GbjToolsInterface"

	^(SystemRepository listInstances: (Array with: aClass)) first asArray
%
category: 'private11-var refs'
method: GbjToolsInterface
instVarNamesInClass: aClass
	^aClass allInstVarNames
%
category: 'private11-misc'
method: GbjToolsInterface
isObject: anObject aKindOf: aClass
	^anObject isKindOf: aClass
%

category: 'private-inspecting'
method: GbjToolsInterface
basicKeysFor: anObject max: max
	"self new keysFor: AllSymbols max: 100"

	| keys |
	keys := OrderedCollection new: max.
	anObject keysAndValuesDo: [:key :value |
		keys addLast: key.
		(keys size == max) ifTrue: [^keys].
         ].
        ^keys
%
category: 'private-inspecting'
method: GbjToolsInterface
keysFor: anObject max: max
	"self new keysFor: AllSymbols max: 100"

	| keys |
        keys := self basicKeysFor: anObject max: max.
        ^(keys asSortedCollection: [:a :b | a printString < b printString]) asArray.
%

category: 'private-browsing'
method: GbjToolsInterface
makeClass: aClass aVersionOf: targetClass
	(targetClass canBeWritten _and: [targetClass classHistory canBeWritten]) ifFalse: [
		^false].
	targetClass addNewVersion: aClass.
	aClass changeNameTo: targetClass name.
	^true
%
category: 'private-browsing'
method: GbjToolsInterface
messagesIn: aSelector class: aClass
	(aSelector == nil) ifTrue: [^Array new].
	(aClass includesSelector: aSelector) ifFalse: [^Array new].
	^(aClass compiledMethodAt: aSelector) _selectorPool asSortedCollection asArray
%
category: 'private11-breakpoints'
method: GbjToolsInterface
methodBreakpoints
	| methods ipOffsets |
	methods := OrderedCollection new.
	ipOffsets := OrderedCollection new.
	GsNMethod _allMethodBreakpoints do: [:each |
		1 to: each size by: 3 do: [:i |
			methods add: (each at: i + 1).
			ipOffsets add: (each at: i + 2)]].
	^Array with: methods with: ipOffsets
%
category: 'private11-browsing'
method: GbjToolsInterface
methodInfoForClass: startClass inProtocol: aProtocol toLevel: startLevel
	^ (self _methodInfoForClass: startClass inProtocol: aProtocol toLevel: startLevel)
%
category: 'private11-browsing'
method: GbjToolsInterface
methodInfoForProtocol: aProtocol inClass: aClass
	^ (self _methodInfoForProtocol: aProtocol class: aClass)
%
category: 'private-browsing'
method: GbjToolsInterface
migrateInstancesOf: fromClass to: toClass
	"Migrate instances of fromClass to toClass."

	| allInstances |
	fromClass == toClass ifTrue: [^self].
	allInstances := self instancesOf: fromClass.
	fromClass migrateInstances: allInstances to: toClass
%
category: 'private-browsing'
method: GbjToolsInterface
moveClass: aClass fromDict: fromDictionary toDict: toDictionary
	| aKey |
	aKey := fromDictionary keyAtValue: aClass.
	toDictionary at: aKey put: aClass.
	fromDictionary removeKey: aKey
	
%
category: 'private-browsing'
method: GbjToolsInterface
moveMethod: aSelector to: newProtocol inClass: aClass
	self addProtocol: newProtocol toClass: aClass.
	(aClass includesSelector: aSelector) ifTrue: [
		aClass moveMethod: aSelector toCategory: newProtocol]
%
category: 'private11-misc'
method: GbjToolsInterface
nameOf: anObject
	^anObject name
%
category: 'private11-misc'
method: GbjToolsInterface
object: anObject at: anIndex
	^anObject at: anIndex
%
category: 'private-inspecting'
method: GbjToolsInterface
objectInfoFor: anObject max: max
	^ (self _objectInfoFor: anObject max: max)
%
category: 'private-inspecting'
method: GbjToolsInterface
objectInfoForNames: names values: values
	^ (self _objectInfoForNames: names values: values)
%
category: 'private-inspecting'
method: GbjToolsInterface
printStringFor: anObject
	| aString |
	aString := anObject printString.
	(aString isKindOf: CharacterCollection) ifFalse: [
		self halt: 'The printString of ' , anObject class name , ' should be a String'].
	^aString
%
category: 'private-browsing'
method: GbjToolsInterface
protocolNamesInClass: aClass
	^ (self _protocolNamesInClass: aClass)
%
category: 'private11-browsing'
method: GbjToolsInterface
protocolNamesInClass: aClass toLevel: aLevel
	^ (self _protocolNamesInClass: aClass toLevel: aLevel)
%
category: 'private11-misc'
method: GbjToolsInterface
protocolOfSelector: aSelector inClass: aClass
	^aClass _categoryOfSelector: aSelector
%
category: 'private-browsing'
method: GbjToolsInterface
recompileWithSubclassesFrom: oldClass to: newClass failedMethods: failedMethods symbolList: symList

	| cvs pds newMeta rfailed src sel cls definition newSubclass subs failedClass |
	cvs := oldClass _classVars keys.
	newClass _classVars keys asArray do: [ :key | (cvs includes: key) ifTrue: [ cvs remove: key ]].
	pds := oldClass sharedPools asOrderedCollection.	
	newClass sharedPools asOrderedCollection do: [ :oop | (pds includes: oop) ifTrue: [ pds remove: oop ]].
	rfailed := newClass _copyMethodsAndVariablesFrom: oldClass except: (Array with: $V with: cvs with: $P with: pds) dictionaries: symList.

	newMeta := newClass class.
	rfailed do: [:failedMethod |
		src := failedMethod _sourceString.
		sel := failedMethod _selector.
		cls := failedMethod _inClass.
		failedMethods add: 'Failed method'.
		failedClass := cls == oldClass ifTrue: [ newClass ] ifFalse: [ newMeta ].
		failedMethods add: failedClass.
		failedMethods add: failedClass name.
		failedMethods add: sel.
		failedMethods add: src.
		failedMethods add: (cls _categoryOfSelector: sel).
		failedMethods add: (cls name , Character space , sel)].

	subs := self classOrganizer subclassesOf: oldClass.

	"Private.  Do this -after- #subclassesOf:, which has the side effect of replacing the new
	  class with the old class in the organizer"
	self classOrganizer addClass: newClass.

	"Iterate over all the first-level subclasses of the old class to create new subclasses"
	subs do: [ :oldSubclass |
		definition := oldSubclass definition.
		newSubclass := self evaluate: definition inContext: nil names: nil values: nil printing: false.
		(newSubclass ~~ nil) ifTrue: [ self
			recompileWithSubclassesFrom: oldSubclass
			to: newSubclass
			failedMethods: failedMethods
			symbolList: symList]]
%
category: 'private11-browsing'
method: GbjToolsInterface
referencesTo: aSymbol
	^ (self _referencesTo: aSymbol)
	
%
category: 'private11-var refs'
method: GbjToolsInterface
referencesToClass: aClass
	^ (self _referencesToClass: aClass)
	
%
category: 'private11-var refs'
method: GbjToolsInterface
referencesToClassVar: cvString startingClass: startClass
	^ (self _referencesToClassVar: cvString startingClass: startClass)
	
%
category: 'private11-var refs'
method: GbjToolsInterface
referencesToInstVar: ivString startingClass: startClass
	^ (self _referencesToInstVar: ivString startingClass: startClass)
	
%
category: 'private11-breakpoints'
method: GbjToolsInterface
removeAllBreaks
	GsNMethod clearAllBreaks
%
category: 'private11-breakpoints'
method: GbjToolsInterface
removeBreakpoint: stepPoint inMethod: aMethod
	aMethod clearBreakAtStepPoint: stepPoint
%
category: 'private-browsing'
method: GbjToolsInterface
removeClass: aClass fromDict: aDictionary
	| aKey |
	aKey := aDictionary keyAtValue: aClass ifAbsent: [nil].
	(aKey ~~ nil) ifTrue: [aDictionary removeKey: aKey]
	
%
category: 'private-browsing'
method: GbjToolsInterface
removeDict: aDict
	System myUserProfile symbolList remove: aDict ifAbsent: []
%
category: 'private-browsing'
method: GbjToolsInterface
removeFromClassHistory: aClass
	aClass classHistory size > 1 ifTrue: [
		aClass classHistory removeVersion: aClass]
%
category: 'private11-misc'
method: GbjToolsInterface
removeProtocol: aProtocol fromClass: aClass
	aClass removeCategory: aProtocol
%
category: 'private-browsing'
method: GbjToolsInterface
removeSelector: aSelector fromClass: aClass
	(aClass includesSelector: aSelector) ifTrue: [
		aClass removeSelector: aSelector]
%
category: 'private-browsing'
method: GbjToolsInterface
renameDict: aDict as: aString
	^System myUserProfile _renameDictionary: aDict to: aString asSymbol
%
category: 'private-browsing'
method: GbjToolsInterface
renameProtocol: oldName to: newName class: aClass
	(aClass categoryNames includes: oldName asSymbol) ifTrue: [
		aClass renameCategory: oldName asSymbol to: newName asSymbol]
%
category: 'private-browsing'
method: GbjToolsInterface
reportClass: aClass indentLevel: indentLevel on: list
	"Create and add to <list>, a class holder suitable for display in a hierarchy browser."

	list add: aClass.
	list add: aClass name.
	list add: (aClass classHistory indexOf: aClass).
	list add: aClass classHistory size.
	list add: indentLevel.
	list add: ' (' , (aClass _instVarNamesWithSeparator: '') , ' )'.
	list add: aClass class
%
category: 'private11-var refs'
method: GbjToolsInterface
reportRefsToClassVar: cvAssoc inClass: inClass on: info
	| aClass |
	aClass := inClass.
	aClass selectors do: [:each |
		| aMethod |
		aMethod := aClass compiledMethodAt: each.
		(aMethod literals includes: cvAssoc) ifTrue: [
			info add: aMethod.
			info add: aMethod _selector.
			info add: aClass.
			info add: aClass name.
			info add: (aClass name , Character space , aMethod _selector).
			info add: (aClass _categoryOfSelector: aMethod _selector)]].
	aClass := inClass class.
	aClass selectors do: [:each |
		| aMethod |
		aMethod := aClass compiledMethodAt: each.
		(aMethod literals includes: cvAssoc) ifTrue: [
			info add: aMethod.
			info add: aMethod _selector.
			info add: aClass.
			info add: aClass name.
			info add: (aClass name , Character space , aMethod _selector).
			info add: (aClass _categoryOfSelector: aMethod _selector)]].

			
%
category: 'private11-var refs'
method: GbjToolsInterface
reportRefsToClassVar: cvAssoc inSubclassesOf: aClass on: info
	| subs |
	subs := self classOrganizer subclassesOf: aClass.
	subs do: [ :subclass |
		self reportRefsToClassVar: cvAssoc inClass: subclass on: info.
		self reportRefsToClassVar: cvAssoc inSubclassesOf: subclass on: info]
%
category: 'private11-var refs'
method: GbjToolsInterface
reportRefsToInstVar: ivName inClass: aClass on: info
	aClass selectors do: [:each |
		| aMethod |
		aMethod := aClass compiledMethodAt: each.
		(aMethod instVarsAccessed includes: ivName) ifTrue: [
			info add: aMethod.
			info add: aMethod _selector.
			info add: aClass.
			info add: aClass name.
			info add: (aClass name , Character space , aMethod _selector).
			info add: (aClass _categoryOfSelector: aMethod _selector)]].

			
%
category: 'private11-var refs'
method: GbjToolsInterface
reportRefsToInstVar: ivName inSubclassesOf: aClass on: info
	| subs |
	subs := self classOrganizer subclassesOf: aClass.
	subs do: [ :subclass |
		self reportRefsToInstVar: ivName inClass: subclass on: info.
		self reportRefsToInstVar: ivName inSubclassesOf: subclass on: info]
%
category: 'private-browsing'
method: GbjToolsInterface
reportSubclassesOf: aClass indentLevel: indentLevel on: list
	| subs |
	subs := self classOrganizer subclassesOf: aClass.
	subs do: [ :subclass |
		self reportClass: subclass indentLevel: indentLevel on: list.
		self reportSubclassesOf: subclass indentLevel: indentLevel + 1 on: list]
%
category: 'private-accessing'
method: GbjToolsInterface
resetClassOrganizer
	classOrganizer := nil
%
category: 'private11-debugging'
method: GbjToolsInterface
selectionIn: source fromStepPoint: aStepPoint offsets: offsets
	| startOffset endOffset found limit |

	aStepPoint > offsets size
	  ifTrue: [
		"Stepping out of an inner activation can leave the steppoint pointing past the end of the offsets array"
		endOffset := source size.
		startOffset := endOffset - 1]
	  ifFalse: [
		startOffset := offsets at: aStepPoint.
		aStepPoint = offsets size
			ifTrue: [limit := source size]
			ifFalse: [limit := offsets at: (aStepPoint + 1)].
		limit < startOffset
			ifTrue: [limit := source size].
		endOffset := startOffset + 1.
		found := false.
		[ (endOffset <= limit) _and: [found not] ] whileTrue: [
			found := (source at: endOffset) isSeparator.
			found ifFalse: [ endOffset := endOffset + 1 ]]].
	^Array with: startOffset with: endOffset - 1
%
category: 'private-browsing'
method: GbjToolsInterface
selectorsForClass: aClass
	^aClass selectors asSortedCollection
%
category: 'private-browsing'
method: GbjToolsInterface
selectorsIn: aProtocol class: aClass
	^ (self _selectorsIn: aProtocol class: aClass)
%
category: 'private-browsing'
method: GbjToolsInterface
senderInfoForSelector: aSelector
	"self new senderInfoForSelector: 'copy'"

	^ (self _senderInfoForSelector: aSelector)
%
category: 'private11-breakpoints'
method: GbjToolsInterface
setBreakIn: aMethod at: textPosition
	| offset |
	offset := self closestOffsetIn: aMethod at: textPosition.
	aMethod setBreakAtStepPoint: offset
%
category: 'private11-misc'
method: GbjToolsInterface
sizeOf: anObject
	^anObject size
%
category: 'private-browsing'
method: GbjToolsInterface
sourceCodeAt: aSelector inClass: aClass
	^(aClass includesSelector: aSelector)
		ifTrue: [aClass sourceCodeAt: aSelector]
		ifFalse: [aClass name , '>>' , aSelector , ' does not exist']
%
category: 'private11-misc'
method: GbjToolsInterface
sourceStringFor: anObject
	^anObject _sourceString
%
category: 'private11-debugging'
method: GbjToolsInterface
stackInfoForProcess: aProcess toLevel: aLevel
	^ (self _stackInfoForProcess: aProcess toLevel: aLevel)
%
category: 'private-debugging'
method: GbjToolsInterface
stackInfoToLevel: aLevel
	^self class stackInfoToLevel: aLevel
%
! 45643: replace _stepInto with _stepOverInFrame: 0
category: 'private11-debugging'
method: GbjToolsInterface
stepIntoInProcess: aProcess
	^aProcess _stepOverInFrame: 0 mode: false replace: false tos: nil
%
category: 'private11-debugging'
method: GbjToolsInterface
stepOverInProcess: aProcess
	^aProcess _stepOver
%
category: 'private11-debugging'
method: GbjToolsInterface
stepOverInProcess: aProcess inFrame: aLevel
	^aProcess _stepOverInFrame: aLevel
%
category: 'private-inspecting'
method: GbjToolsInterface
storeObject: slotObject intoDictionary: anObject atKey: aKey
	anObject at: aKey put: slotObject
%
category: 'private-inspecting'
method: GbjToolsInterface
storeObject: slotObject intoNSC: anObject
	"Don't do anything in this case."
%
category: 'private-inspecting'
method: GbjToolsInterface
storeObject: slotObject intoObject: anObject atIndex: slot atKey: aKey isNamedSlot: isNamedSlot 
	isNamedSlot ifTrue: [
		anObject instVarAt: slot put: slotObject.
		^self].
	(anObject isKindOf: AbstractDictionary) ifTrue: [
		^self storeObject: slotObject intoDictionary: anObject atKey: aKey].
	(anObject isKindOf: UnorderedCollection) ifTrue: [
		^self storeObject: slotObject intoNSC: anObject].
	anObject basicAt: slot put: slotObject
%
category: 'private-browsing'
method: GbjToolsInterface
subclassesOf: aClass
	| subs |
	subs := self classOrganizer subclassesOf: aClass.
	subs := subs asSortedCollection: [:a :b | a name < b name].
	^subs asArray
%
category: 'private11-browsing'
method: GbjToolsInterface
substringSearch: aString

	^ (self _substringSearch: aString)
%
category: 'private-browsing'
method: GbjToolsInterface
superclassChainFor: gsclass 
	| result this superCls |
	result := OrderedCollection new: 5.
	this := gsclass.	
	[(superCls := this superclass) == nil] whileFalse:  [
		result addFirst: superCls.
		this := superCls].
	^result
%
category: 'private-executing'
method: GbjToolsInterface
symbolListWithNames: names values: values
	| symList tempDict val |
	^(names == nil)
		ifTrue: [System myUserProfile symbolList]
		ifFalse: [
			symList := SymbolList new.
			tempDict := SymbolDictionary new.
			1 to: names size do: [ :i |
				val := i > values size ifTrue: [nil] ifFalse: [values at: i].
				tempDict at: (names at: i) asSymbol put: val].
			symList add: tempDict.
			symList addAll: System myUserProfile symbolList.
			symList]
%
category: 'private-searching'
method: GbjToolsInterface
symbolsMatching: aString
	"Private.  Return a list of Symbols that match <aString> which may contain wild-card characters * and ?."

	| matches pattern |
	matches := OrderedCollection new.
	pattern := self asGSPattern: aString.
	((AllUsers userWithId: #SymbolUser) resolveSymbol: #AllSymbols) do: [:each | (each matchPattern: pattern) ifTrue: [matches add: each]].
	^matches asSortedCollection asArray
%
category: 'private11-misc'
method: GbjToolsInterface
thisClass: aClass
	^aClass thisClass
%
category: 'private11-debugging'
method: GbjToolsInterface
trimStackOf: aProcess toLevel: aLevel
	^aProcess _trimStackToLevel: (self initialInvocationOfMethodAtLevel: aLevel)
%
category: 'private-inspecting'
method: GbjToolsInterface
valuesFor: anObject max: max
	"self new valuesFor: AllSymbols max: 100"

	| values |
	values := OrderedCollection new: max.
	anObject do: [:value |
		values addLast: value.
		values size = max ifTrue: [
			^(values asSortedCollection: [:a :b | a printString < b printString]) asArray]].
	^(values asSortedCollection: [:a :b | a printString < b printString]) asArray
%
category: 'private-browsing'
method: GbjToolsInterface
versionedOldClass: oldClass withNewClass: newClass info: info
	self
		recompileWithSubclassesFrom: oldClass
		to: newClass
		failedMethods: info
		symbolList: System myUserProfile symbolList
%
category: 'private-filing out'
method: GbjToolsInterface
writeFileOutHeaderOn: aFile 

	"This method will write a fileout header onto the given file."

	| rawVer beVer lf |
	rawVer := System _version.
	beVer := ''.
	lf := String with: Character lf.
	"Comment each newline"
	(self asArrayOfLines: rawVer) do: [:line | beVer := beVer, '! ', line, lf].
	aFile
		nextPutAll: '!'; lf;
		nextPutAll: '! From ';
		nextPutAll: beVer; lf;
		nextPutAll: '! On ';
		nextPutAll: Date today printString;
		nextPutAll: ', ';
		nextPutAll: Time now printString; lf;
		nextPutAll: '!'; lf
%
category: 'private-browsing'
method: GbjToolsInterface
_acceptClassText: aString oldClass: oldClass
	| newClass info |
	newClass := aString evaluate.
	info := OrderedCollection new.

	"Make sure what we compiled to a class object."
	(newClass isKindOf: Class) ifFalse: [
		info add: 'Not a class'.
		info add: newClass.
		^info].

	"If the oldClass is a class and the old and new class share a class history, then copy methods, etc.
	to the new class and ask about migrating instances.  Otherwise, just update the class organizer
	with the new class."
	info add: 'New class'.
	info add: newClass.
	info add: newClass name.
	((oldClass ~~ nil) _and: [newClass classHistory == oldClass classHistory])
		ifTrue:  [self versionedOldClass: oldClass withNewClass: newClass info: info]
		ifFalse: [self classOrganizer addClass: newClass].

	^info
%
category: 'private11-breakpoints'
method: GbjToolsInterface
_breakpointInfo
	"self new _breakpointInfo"

	| anArray methods ipOffsets info aMethod ipOffset aClass stepPoint range state |
	info := OrderedCollection new.
	anArray := self methodBreakpoints.
	methods := anArray at: 1.
	ipOffsets := anArray at: 2.
	1 to: methods size do: [:i |
		aMethod := methods at: i.
		ipOffset := ipOffsets at: i.
		stepPoint := aMethod _nextStepPointForIp: ipOffset.
		aClass := aMethod _inClass.
		info add: aMethod.
		info add: aMethod _selector.
		info add: aClass.
		info add: aClass name.
		state := ipOffset negative ifTrue: [' (disabled)'] ifFalse: [''].
		info add: (aClass name , Character space , aMethod _selector , ' @ ' , stepPoint printString , state).
		info add: (aClass _categoryOfSelector: aMethod _selector).
		range := self selectionIn: aMethod _sourceString fromStepPoint: stepPoint offsets: aMethod _sourceOffsets.
		info add: range first.
		info add: range last.
		info add: stepPoint].
	^info
%
category: 'private-browsing'
method: GbjToolsInterface
_classInfoForDictionary: aDictionary
	| classes classInfo |
	classes := OrderedCollection new.
	classInfo := OrderedCollection new.
	aDictionary associationsDo: [:each |
		each value isBehavior ifTrue: [
			classes add: each value]].
		classes := (classes asSortedCollection: [:a :b | a name < b name]) asArray.
		classes do: [:each |
			self classInfoForClass: each into: classInfo].
	^classInfo
%
! 40314 addition
category: 'private-browsing'
method: GbjToolsInterface
classNamesForDictionary: aDictionary
	| classes classInfo |
	classes := OrderedCollection new.
	classInfo := OrderedCollection new.
	aDictionary associationsDo: [:each |
		each value isBehavior ifTrue: [
			classes add: each value]].
		classes := (classes asSortedCollection: [:a :b | a name < b name]) asArray.
        ^ classes collect: [:class | class asString ]
%
category: 'private-browsing'
method: GbjToolsInterface
_classVersionInfoForHistory: hist
	| classes classInfo |
	classes := OrderedCollection new.
	classInfo := OrderedCollection new.
	hist do: [:each | classes addFirst: each].
	classes do: [:each |
		classInfo add: each.
		classInfo add: each name.
		classInfo add: (hist indexOf: each).
		classInfo add: hist size.
		classInfo add: each class].
	^classInfo
%
category: 'private-browsing'
method: GbjToolsInterface
_compile: aString class: aClass category: aCategory
	"self new compile: 'foo
		| unused |
		^#foo' class: GbjToolsInterface category: #unused "

	| aSegment aMethodOrError result warningString errorArray |
        warningString := ''.
	aSegment := System currentSegment .
	System currentSegment: aClass segment .  "switch to the Class's segment"
        [[ aMethodOrError := aClass compileMethod: aString 
                 dictionaries: System myUserProfile symbolList
                 category: aCategory asSymbol environmentId: 0.
         ] onException: CompileError do: [ :ex |
              aMethodOrError := Array new.
              aMethodOrError at: 1 put: (ex asString).
              aMethodOrError at: 2 put: (ex errorDetails).
              ]] onException: CompileWarning do: [:ex |
                warningString := ex _legacyHandlerArgs at: 1.
                ex resume.
                ].
        
	System currentSegment: aSegment.
	result := OrderedCollection new.

	aMethodOrError _class == GsNMethod
		ifTrue:	[
			aString immediateInvariant.
			result add: true.
			result add: aMethodOrError.
			result add: aMethodOrError selector.
                        result add: warningString]
		ifFalse: [
			errorArray := aMethodOrError at: 2.
			result add: false.
			result add: ((GemStoneError at: #English) at: 
                                     ((errorArray at: 1) at: 1)).
			result add: ((errorArray at: 1) at: 2)]	.
	^result
%

category: 'private-browsing'
method: GbjToolsInterface
_dictionaryInfo
	| dictionaries dictionaryInfo |
	dictionaries := System myUserProfile symbolList asArray.
	dictionaryInfo := OrderedCollection new.
	dictionaries do: [:each |
		dictionaryInfo add: each.
		dictionaryInfo add: each name].
	^dictionaryInfo
%
category: 'private-browsing'
method: GbjToolsInterface
_dictionaryInfoForClass: aClass
	"GbjToolsInterface dictionaryInfoForClass: Object"

	| dict arr |
	arr := System myUserProfile dictionaryAndSymbolOf: aClass.
	^((arr == nil)
		ifTrue: [Array new] 
		ifFalse: [
			dict := arr at: 1. 
			Array with: dict name with: dict])
%
category: 'private-browsing'
method: GbjToolsInterface
_errorInfoFrom: errorArray
	| firstError errorNumber sourceIndex errorText |
	firstError := errorArray at: 1.
	errorNumber := firstError at: 1.
	sourceIndex := firstError at: 2.
	errorText := (GemStoneError at: System myUserProfile nativeLanguage) at: errorNumber.
	^Array with: errorText with: sourceIndex
%
category: 'private-browsing'
method: GbjToolsInterface
_hierarchyListForClass: aClass
	| list indentLevel   |
	list := OrderedCollection new.
	indentLevel := 0.
	"Superclasses"
	(self superclassChainFor: aClass) do: [ :gsSuper |
		self reportClass: gsSuper indentLevel: indentLevel on: list.
		indentLevel := indentLevel + 1].
	"Target class"
	self reportClass: aClass indentLevel: indentLevel on: list.
	"Subclasses"
	self reportSubclassesOf: aClass indentLevel: indentLevel + 1 on: list.
	^list
%
category: 'private-browsing'
method: GbjToolsInterface
_implementorInfoForSelector: aSelector
	"self new implementorInfoForSelector: 'copy'"

	| implementors info aMethod aClass |
	info := OrderedCollection new.
	(aSelector == nil) ifTrue: [^info].
	implementors := self classOrganizer implementorsOf: aSelector asSymbol.
	1 to: implementors size do: [:i |
		aMethod := implementors at: i.
		aClass := aMethod _inClass.
		info add: aMethod.
		info add: aMethod _selector.
		info add: aClass.
		info add: aClass name.
		info add: (aClass name , Character space , aMethod _selector).
		info add: (aClass _categoryOfSelector: aMethod _selector)].
	^info
%
category: 'private-browsing'
method: GbjToolsInterface
_infoForClassNamed: aString
	| aClass |
	aClass := System myUserProfile objectNamed: aString asSymbol.
	^(aClass isBehavior
		ifTrue: [self classInfoForClass: aClass]
		ifFalse: [Array new])
%
category: 'private11-browsing'
method: GbjToolsInterface
_methodInfoForClass: startClass inProtocol: aProtocol toLevel: startLevel
	"self new _methodInfoForClass: Array inProtocol: nil toLevel: -1"

	| hierarchy aLevel info selectors |
	hierarchy := startLevel = -1
			ifTrue: [aLevel := 1.  OrderedCollection with: startClass]
			ifFalse: [aLevel := startLevel.  (self superclassChainFor: startClass) copyWith: startClass].
	info := OrderedCollection new.
	aLevel to: hierarchy size do: [:i |
		| aClass |
		aClass := hierarchy at: i.
		(aProtocol == nil)
			ifTrue: [selectors := aClass selectors]
			ifFalse: [(aClass categoryNames includes: aProtocol asSymbol)
				ifTrue: [selectors := aClass selectorsIn: aProtocol asSymbol]
				ifFalse: [selectors := Array new]].
		selectors do: [:each |
			| aMethod methInfo printString |
			aMethod := aClass compiledMethodAt: each.
			methInfo := OrderedCollection new.
			info add: methInfo.
			"aClass := aMethod _inClass."
			methInfo add: aMethod.
			methInfo add: aMethod _selector.
			methInfo add: aClass.
			methInfo add: aClass name.
			printString := aMethod _selector.
			aClass ~~ startClass ifTrue: [printString := printString , ' (', aClass name , ')'].
			methInfo add: printString.
			methInfo add: (aClass _categoryOfSelector: aMethod _selector)]].
	info := info asSortedCollection: [:a :b | (a at: 2) <= (b at: 2)].
	^info inject: OrderedCollection new into: [:list :each | list , each]
%
category: 'private11-browsing'
method: GbjToolsInterface
_methodInfoForProtocol: aProtocol class: aClass
	| info selectors |
	((aProtocol ~~ nil) _and: [(aClass categoryNames includes: aProtocol asSymbol) not]) ifFalse: [^Array new].
	(aProtocol ~~ nil) 
		ifTrue: [selectors := (aClass selectorsIn: aProtocol asSymbol) asSortedCollection asArray]
		ifFalse: [selectors := aClass selectors asSortedCollection asArray].
	info := OrderedCollection new.
	selectors do: [:each |
		| aMethod |
		aMethod := aClass compiledMethodAt: each.
		"aClass := aMethod _inClass."
		info add: aMethod.
		info add: aMethod _selector.
		info add: aClass.
		info add: aClass name.
		info add: (aClass name , Character space , aMethod _selector).
		info add: (aClass _categoryOfSelector: aMethod _selector)].
	^info
%
category: 'private11-browsing'
method: GbjToolsInterface
_methodInfoForProtocol: aProtocol class: aClass toLevel: aLevel
	| info selectors |
	((aProtocol ~~ nil) _and: [(aClass categoryNames includes: aProtocol asSymbol) not]) ifFalse: [^Array new].
	(aProtocol ~~ nil) 
		ifTrue: [selectors := (aClass selectorsIn: aProtocol asSymbol) asSortedCollection asArray]
		ifFalse: [selectors := aClass selectors asSortedCollection asArray].
	info := OrderedCollection new.
	selectors do: [:each |
		| aMethod |
		aMethod := aClass compiledMethodAt: each.
		"aClass := aMethod _inClass."
		info add: aMethod.
		info add: aMethod _selector.
		info add: aClass.
		info add: aClass name.
		info add: (aClass name , Character space , aMethod _selector).
		info add: (aClass _categoryOfSelector: aMethod _selector)].
	^info
%
category: 'private-inspecting'
method: GbjToolsInterface
_objectInfoFor: anObject max: max
	| info ivNames namedSize idxSize size obj |
	(anObject isKindOf: AbstractDictionary) ifTrue: [^self _objectInfoForDictionary: anObject max: max].
	(anObject isKindOf: UnorderedCollection) ifTrue: [^self _objectInfoForNSC: anObject max: max].
	info := OrderedCollection new.
	info add: 0.
	info add: 0.
	info add: 'self'.
	info add: anObject.
	info add: anObject class name.
	ivNames := anObject class allInstVarNames.
	namedSize := anObject class instSize.
	info add: namedSize.
	1 to: namedSize do: [:i |
		info add: i.
		info add: i.
		info add: (ivNames at: i).
		info add: (obj := anObject instVarAt: i).
		info add: obj class name].
	size := anObject basicSize - anObject class instSize.
	idxSize := size min: max.
	info add: idxSize.
	1 to: idxSize do: [:i |
		info add: i.
		info add: i.
		info add: i printString.
		info add: (obj := anObject basicAt: i).
		info add: obj class name].
	info add: size.
	^info
%
category: 'private-inspecting'
method: GbjToolsInterface
_objectInfoForDictionary: anObject max: max
	| info ivNames namedSize idxSize size obj keys keyStrings |
	info := OrderedCollection new.
	info add: 0.
	info add: 0.
	info add: 'self'.
	info add: anObject.
	info add: anObject class name.
	ivNames := anObject class allInstVarNames.
	namedSize := anObject class instSize.
	info add: namedSize.
	1 to: namedSize do: [:i |
		info add: i.
		info add: i.
		info add: (ivNames at: i).
		info add: (obj := anObject instVarAt: i).
		info add: obj class name].
	keys := self keysFor: anObject max: max.
	keyStrings := keys collect: [:each | each printString].
	size := anObject size.
	idxSize := keys size.
	info add: idxSize.
	1 to: idxSize do: [:i |
		info add: i.
		info add: (keys at: i).
		info add: (keyStrings at: i).
		info add: (obj := anObject at: (keys at: i)).
		info add: obj class name].
	info add: size.
	^info
%
category: 'private-inspecting'
method: GbjToolsInterface
_objectInfoForNames: names values: values
	| info obj |
	info := OrderedCollection new.
	1 to: names size do: [:i |
		info add: i.
		info add: i.
		info add: (names at: i).
		obj := i > values size ifTrue: [nil] ifFalse: [values at: i].
		info add: obj.
		info add: obj class name].
	^info
%
category: 'private-inspecting'
method: GbjToolsInterface
_objectInfoForNSC: anObject max: max
	| info ivNames namedSize idxSize size obj elements elementStrings |
	info := OrderedCollection new.
	info add: 0.
	info add: 0.
	info add: 'self'.
	info add: anObject.
	info add: anObject class name.
	ivNames := anObject class allInstVarNames.
	namedSize := anObject class instSize.
	info add: namedSize.
	1 to: namedSize do: [:i |
		info add: i.
		info add: i.
		info add: (ivNames at: i).
		info add: (obj := anObject instVarAt: i).
		info add: obj class name].
	elements := self valuesFor: anObject max: max.
	elementStrings := elements collect: [:each | each printString].
	size := anObject size.
	idxSize := elements size.
	info add: idxSize.
	1 to: idxSize do: [:i |
		info add: i.
		info add: (elements at: i).
		info add: (elementStrings at: i).
		info add: (obj := elements at: i).
		info add: obj class name].
	info add: size.
	^info
%
category: 'private-browsing'
method: GbjToolsInterface
_protocolNamesInClass: aClass
	^aClass categoryNames asSortedCollection asArray
%
category: 'private11-browsing'
method: GbjToolsInterface
_protocolNamesInClass: startClass toLevel: startLevel
	"self new _protocolNamesInClass: Array toLevel: 2"

	| hierarchy aLevel info |
	hierarchy := startLevel = -1
			ifTrue: [aLevel := 1.  OrderedCollection with: startClass]
			ifFalse: [aLevel := startLevel.  (self superclassChainFor: startClass) copyWith: startClass].
	info := Set new.
	aLevel to: hierarchy size do: [:i |
		| aClass |
		aClass := hierarchy at: i.
		info addAll: aClass categoryNames].
	^info asSortedCollection asArray

%
category: 'private11-browsing'
method: GbjToolsInterface
_referencesTo: aString
	"self new _referencesTo: #ClassOrganizer"
	| anArray methods positions info aMethod aPosition aClass |
	info := OrderedCollection new.
	anArray := self classOrganizer referencesTo: aString asSymbol.
	methods := anArray at: 1.
	positions := anArray at: 2.
	1 to: methods size do: [:i |
		aMethod := methods at: i.
		aPosition := positions at: i.
		aClass := aMethod _inClass.
		info add: aMethod.
		info add: aMethod _selector.
		info add: aClass.
		info add: aClass name.
		info add: (aClass name , Character space , aMethod _selector).
		info add: (aClass _categoryOfSelector: aMethod _selector).
		info add: aPosition].
	^info
%
category: 'private11-var refs'
method: GbjToolsInterface
_referencesToClass: refClass
	"self new _referencesToClass: DateTime"

	| info anArray cAssoc classSet |
	info := OrderedCollection new.
	anArray := System myUserProfile dictionaryAndSymbolOf: refClass.
	(anArray == nil) ifTrue: [^info].
	cAssoc := (anArray at: 1) associationAt: (anArray at: 2).
        classSet := self classOrganizer classes.
	1 to: classSet size do: [:n| |each aClass selectors| 
		each := classSet at: n.
		aClass := each.
                selectors := aClass selectors.
		1 to: selectors size  do: [:m| |sel aMethod |
                        sel := selectors at: m.
			aMethod := aClass compiledMethodAt: sel.
			(aMethod literals includes: cAssoc) ifTrue: [
				info add: aMethod.
				info add: aMethod _selector.
				info add: aClass.
				info add: aClass name.
				info add: (aClass name , Character space , aMethod _selector).
				info add: (aClass _categoryOfSelector: aMethod _selector)]].
		aClass := each class.
                selectors := aClass selectors.
		1 to: selectors size  do: [:m| |sel aMethod |
                        sel := selectors at: m.
			aMethod := aClass compiledMethodAt: sel.
			(aMethod literals includes: cAssoc) ifTrue: [
				info add: aMethod.
				info add: aMethod _selector.
				info add: aClass.
				info add: aClass name.
				info add: (aClass name , Character space , aMethod _selector).
				info add: (aClass _categoryOfSelector: aMethod _selector)]]].


	^info
	
	
	
%
category: 'private11-var refs'
method: GbjToolsInterface
_referencesToClassVar: cvString startingClass: startClass
	"self new _referencesToClassVar: 'MonthNames' startingClass: DateTime"

	| cvName cvAssoc definingClass list |
	cvName := cvString asSymbol.
	list := OrderedCollection new.
	((self superclassChainFor: startClass) copyWith: startClass) do: [:each |
		(each classVarNames includes: cvName) ifTrue: [
			definingClass := each]].
	(definingClass == nil) ifTrue: [^list].
	cvAssoc := definingClass _resolveClassVar: cvName.
	self reportRefsToClassVar: cvAssoc inClass: definingClass on: list.
	self reportRefsToClassVar: cvAssoc inSubclassesOf: definingClass on: list.
	^list
	
	
	
%
category: 'private11-var refs'
method: GbjToolsInterface
_referencesToInstVar: ivString startingClass: startClass
	"self new _referencesToInstVar: 'exportSet' startingClass: GbjTransportBuffer"

	| ivName definingClass list |
	ivName := ivString asSymbol.
	list := OrderedCollection new.
	((self superclassChainFor: startClass) copyWith: startClass) do: [:each |
		(each instVarNames includes: ivName) ifTrue: [
			definingClass := each]].
	(definingClass == nil) ifTrue: [^list].
	self reportRefsToInstVar: ivName inClass: definingClass on: list.
	self reportRefsToInstVar: ivName inSubclassesOf: definingClass on: list.
	^list
	
	
	
%
category: 'private-browsing'
method: GbjToolsInterface
_selectorsIn: aProtocol class: aClass
	^((aClass categoryNames includes: aProtocol asSymbol)
		ifTrue: [(aClass selectorsIn: aProtocol asSymbol) asSortedCollection asArray]
		ifFalse: [Array new])
%
category: 'private-browsing'
method: GbjToolsInterface
_senderInfoForSelector: aSelector
	"self new senderInfoForSelector: 'copy'"

	| anArray senders positions info aMethod aPosition aClass |
	info := OrderedCollection new.
	(aSelector == nil) ifTrue: [^info].
	anArray := self classOrganizer sendersOf: aSelector asSymbol.
	senders := anArray at: 1.
	positions := anArray at: 2.
	1 to: senders size do: [:i |
		aMethod := senders at: i.
		aPosition := positions at: i.
		aClass := aMethod _inClass.
		info add: aMethod.
		info add: aMethod _selector.
		info add: aClass.
		info add: aClass name.
		info add: (aClass name , Character space , aMethod _selector).
		info add: (aClass _categoryOfSelector: aMethod _selector).
		info add: aPosition].
	^info
%
category: 'private11-debugging'
method: GbjToolsInterface
_stackInfoForProcess: aProcess fromLevel: startLevel toLevel: aLevel
	" Private - Answer an array of information about the GemStone execution state of the
	  process in aGsError.  The first entry is another array (see GS class GsProcess, method
	  #_gsiStackReportFromLevel:toLevel:, for more details).  The second entry is an array
	  containing selected fields from the first entry that we know we will need as local
	  objects (this is a performance hack to reduce round-trips to GemStone). "
	"GbjToolsInterface _stackInfoToLevel: 10"

	| className aProtocol aSelector aMethod methodClass frameId fullEntry stackReport stackReportExtract quickStepPoint sourceOffsets sourceString range |
	stackReport := aProcess _gsiStackReportFromLevel: startLevel toLevel: aLevel.
	stackReportExtract := OrderedCollection new: stackReport size * 10.
	1 to: stackReport size / 2 do: [:index |
		frameId := stackReport at: index * 2 - 1.
		fullEntry := stackReport at: index * 2.
		aMethod := fullEntry at: 1.
		methodClass := aMethod _inClass.
		"methodClass == GsProcess ifTrue: [^stackReportExtract]."
		stackReportExtract add: frameId.
		stackReportExtract add: aMethod. "gsMethod"
		stackReportExtract add: methodClass. "method class"
		className := (methodClass == nil) ifTrue: [#Unknown] ifFalse: [methodClass name].
		stackReportExtract add: className.
		stackReportExtract add: (fullEntry at: 3). "self"
		aSelector := fullEntry at: 4.
		stackReportExtract add: aSelector. "selector"
		aProtocol := (methodClass == nil) ifTrue: [#Unknown] ifFalse: [methodClass _categoryOfSelector: aSelector].
		stackReportExtract add: aProtocol.
		quickStepPoint := fullEntry at: 5.
		(quickStepPoint == nil) ifTrue: [
			quickStepPoint := aMethod _stepPointForIp: (fullEntry at: 10) level: index * 2 quick: false].
		sourceOffsets := fullEntry at: 6.
		sourceString := fullEntry at: 9.
                ((quickStepPoint == nil) _or: [quickStepPoint = 0 _or: [(sourceOffsets == nil) _or: [sourceOffsets isEmpty]]])
                    ifTrue: [ stackReportExtract add: -1.  stackReportExtract add: -1]
                    ifFalse: [
			range := self selectionIn: sourceString fromStepPoint: quickStepPoint offsets: sourceOffsets.
			stackReportExtract add: range first.  stackReportExtract add: range last].
		stackReportExtract add: (fullEntry at: 7). "argAndTempNames"
		stackReportExtract add: (fullEntry at: 8)]. "argAndTempValues"
	^stackReportExtract
%
category: 'private11-debugging'
method: GbjToolsInterface
_stackInfoForProcess: aProcess toLevel: aLevel
	| stackInfo |
	stackInfo := self _stackInfoForProcess: aProcess fromLevel: 1 toLevel: aLevel.
	^stackInfo
%
category: 'private-debugging'
method: GbjToolsInterface
_stackInfoToLevel: aLevel
	^self class _stackInfoToLevel: aLevel
%
category: 'private11-browsing'
method: GbjToolsInterface
_substringSearch: aString
	"self new _substringSearch: 'ClassOrganizer'"
	| anArray methods positions info aMethod aPosition aClass |
	info := OrderedCollection new.
	anArray := self classOrganizer substringSearch: aString.
	methods := anArray at: 1.
	positions := anArray at: 2.
	1 to: methods size do: [:i |
		aMethod := methods at: i.
		aPosition := positions at: i.
		aClass := aMethod _inClass.
		info add: aMethod.
		info add: aMethod _selector.
		info add: aClass.
		info add: aClass name.
		info add: (aClass name , Character space , aMethod _selector).
		info add: (aClass _categoryOfSelector: aMethod _selector).
		info add: aPosition].
	^info
%
category: 'private30'
method: GbjToolsInterface
packStrings: arrayOfStrings

	" Pack an array of strings into a single string with \0 separators "
	" First 2 chars are really a 16-bit integer indicating # of strings "
	| str size chr0 |
	chr0 := Character withValue: 0.
	str := String new.
	size := arrayOfStrings size.
	str add: (Character withValue: (size // 256)).
	str add: (Character withValue: (size \\ 256)).
	arrayOfStrings do: [:s | str addAll: s. str add: chr0].
	^ str
%
